package mekanism.client.render.tileentity;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import mekanism.api.MekanismConfig;
import mekanism.client.model.IModelSeismicVibrator;
import mekanism.client.model.LegacyModelSeismicVibrator;
import mekanism.client.model.ModelSeismicVibrator;
import mekanism.common.tile.TileEntitySeismicVibrator;
import mekanism.common.util.MekanismUtils;
import mekanism.common.util.MekanismUtils.ResourceType;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraft.tileentity.TileEntity;
import org.lwjgl.opengl.GL11;

@SideOnly(Side.CLIENT)
public class RenderSeismicVibrator extends TileEntitySpecialRenderer {
    private IModelSeismicVibrator model = MekanismConfig.client.modelType.createModel(
        ModelSeismicVibrator::new, LegacyModelSeismicVibrator::new
    );

    @Override
    public void renderTileEntityAt(
        TileEntity tileEntity, double x, double y, double z, float partialTick
    ) {
        renderAModelAt((TileEntitySeismicVibrator) tileEntity, x, y, z, partialTick);
    }

    private void renderAModelAt(
        TileEntitySeismicVibrator tileEntity,
        double x,
        double y,
        double z,
        float partialTick
    ) {
        GL11.glPushMatrix();
        GL11.glTranslatef((float) x + 0.5F, (float) y + 1.5F, (float) z + 0.5F);

        bindTexture(MekanismUtils.getResource(
            ResourceType.RENDER, model.getTextureNameForState(tileEntity.isActive)
        ));

        switch (tileEntity.facing) {
            case 2:
                GL11.glRotatef(0, 0.0F, 1.0F, 0.0F);
                break;
            case 3:
                GL11.glRotatef(180, 0.0F, 1.0F, 0.0F);
                break;
            case 4:
                GL11.glRotatef(90, 0.0F, 1.0F, 0.0F);
                break;
            case 5:
                GL11.glRotatef(270, 0.0F, 1.0F, 0.0F);
                break;
        }

        float actualRate = (float) Math.sin(
            (tileEntity.clientPiston + (tileEntity.isActive ? partialTick : 0)) / 5F
        );

        GL11.glRotatef(180F, 0.0F, 0.0F, 1.0F);
        model.renderWithPiston(Math.max(0, actualRate), 0.0625F);
        GL11.glPopMatrix();
    }
}
