package mekanism.generators.common.tile.turbine;

import java.util.EnumSet;

import cpw.mods.fml.common.Optional.Interface;
import cpw.mods.fml.common.Optional.InterfaceList;
import cpw.mods.fml.common.Optional.Method;
import ic2.api.energy.EnergyNet;
import ic2.api.energy.event.EnergyTileLoadEvent;
import ic2.api.energy.event.EnergyTileUnloadEvent;
import ic2.api.energy.tile.IEnergyConductor;
import ic2.api.energy.tile.IEnergyTile;
import mekanism.api.Coord4D;
import mekanism.api.MekanismConfig.general;
import mekanism.common.base.IEnergyWrapper;
import mekanism.common.tile.TileEntityGasTank.GasMode;
import mekanism.common.util.CableUtils;
import mekanism.common.util.LangUtils;
import mekanism.common.util.MekanismUtils;
import mekanism.common.util.PipeUtils;
import mekanism.generators.common.content.turbine.TurbineFluidTank;
import net.minecraft.tileentity.TileEntity;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.common.util.ForgeDirection;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidRegistry;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidTankInfo;
import net.minecraftforge.fluids.IFluidHandler;

@InterfaceList({
    @Interface(iface = "ic2.api.energy.tile.IEnergySink", modid = "IC2")
    , @Interface(iface = "ic2.api.energy.tile.IEnergySource", modid = "IC2"),
        @Interface(iface = "ic2.api.tile.IEnergyStorage", modid = "IC2")
})
public class TileEntityTurbineValve
    extends TileEntityTurbineCasing implements IFluidHandler, IEnergyWrapper {
    public boolean ic2Registered = false;

    public TurbineFluidTank fluidTank;

    public TileEntityTurbineValve() {
        super("TurbineValve");
        fluidTank = new TurbineFluidTank(this);
    }

    @Override
    public void onUpdate() {
        super.onUpdate();

        if (!ic2Registered && MekanismUtils.useIC2()) {
            register();
        }

        if (!worldObj.isRemote) {
            if (structure != null) {
                double prev = getEnergy();
                CableUtils.emit(this);
            }
        }
    }

    @Override
    public EnumSet<ForgeDirection> getOutputtingSides() {
        if (structure != null) {
            EnumSet set = EnumSet.allOf(ForgeDirection.class);
            set.remove(ForgeDirection.UNKNOWN);

            for (ForgeDirection side : ForgeDirection.VALID_DIRECTIONS) {
                if (structure.locations.contains(Coord4D.get(this).getFromSide(side))) {
                    set.remove(side);
                }
            }

            return set;
        }

        return EnumSet.noneOf(ForgeDirection.class);
    }

    @Override
    public EnumSet<ForgeDirection> getConsumingSides() {
        return EnumSet.noneOf(ForgeDirection.class);
    }

    @Override
    public boolean canUpdate() {
        return true;
    }

    @Method(modid = "IC2")
    public void register() {
        if (!worldObj.isRemote) {
            TileEntity registered
                = EnergyNet.instance.getTileEntity(worldObj, xCoord, yCoord, zCoord);

            if (registered != this) {
                if (registered instanceof IEnergyTile) {
                    MinecraftForge.EVENT_BUS.post(new EnergyTileUnloadEvent((IEnergyTile
                    ) registered));
                } else if (registered == null) {
                    MinecraftForge.EVENT_BUS.post(new EnergyTileLoadEvent(this));
                    ic2Registered = true;
                }
            }
        }
    }

    @Method(modid = "IC2")
    public void deregister() {
        if (!worldObj.isRemote) {
            TileEntity registered
                = EnergyNet.instance.getTileEntity(worldObj, xCoord, yCoord, zCoord);

            if (registered instanceof IEnergyTile) {
                MinecraftForge.EVENT_BUS.post(new EnergyTileUnloadEvent((IEnergyTile
                ) registered));
            }
        }
    }

    @Override
    public double getMaxOutput() {
        return structure != null ? structure.getEnergyCapacity() : 0;
    }

    @Override
    public void onAdded() {
        super.onAdded();

        if (MekanismUtils.useIC2()) {
            register();
        }
    }

    @Override
    public void onChunkUnload() {
        if (MekanismUtils.useIC2()) {
            deregister();
        }

        super.onChunkUnload();
    }

    @Override
    public void invalidate() {
        super.invalidate();

        if (MekanismUtils.useIC2()) {
            deregister();
        }
    }

    @Override
    public int receiveEnergy(ForgeDirection from, int maxReceive, boolean simulate) {
        return 0;
    }

    @Override
    public int extractEnergy(ForgeDirection from, int maxExtract, boolean simulate) {
        if (getOutputtingSides().contains(from)) {
            double toSend = Math.min(
                getEnergy(), Math.min(getMaxOutput(), maxExtract * general.FROM_TE)
            );

            if (!simulate) {
                setEnergy(getEnergy() - toSend);
            }

            return (int) Math.round(toSend * general.TO_TE);
        }

        return 0;
    }

    @Override
    public boolean canConnectEnergy(ForgeDirection from) {
        return structure != null;
    }

    @Override
    public int getEnergyStored(ForgeDirection from) {
        return (int) Math.round(getEnergy() * general.TO_TE);
    }

    @Override
    public int getMaxEnergyStored(ForgeDirection from) {
        return (int) Math.round(getMaxEnergy() * general.TO_TE);
    }

    @Override
    @Method(modid = "IC2")
    public int getSinkTier() {
        return 4;
    }

    @Override
    @Method(modid = "IC2")
    public int getSourceTier() {
        return 4;
    }

    @Override
    @Method(modid = "IC2")
    public void setStored(int energy) {
        setEnergy(energy * general.FROM_IC2);
    }

    @Override
    @Method(modid = "IC2")
    public int addEnergy(int amount) {
        return (int) Math.round(getEnergy() * general.TO_IC2);
    }

    @Override
    @Method(modid = "IC2")
    public boolean isTeleporterCompatible(ForgeDirection side) {
        return canOutputTo(side);
    }

    @Override
    public boolean canOutputTo(ForgeDirection side) {
        return getOutputtingSides().contains(side);
    }

    @Override
    @Method(modid = "IC2")
    public boolean acceptsEnergyFrom(TileEntity emitter, ForgeDirection direction) {
        return false;
    }

    @Override
    @Method(modid = "IC2")
    public boolean emitsEnergyTo(TileEntity receiver, ForgeDirection direction) {
        return getOutputtingSides().contains(direction)
            && receiver instanceof IEnergyConductor;
    }

    @Override
    @Method(modid = "IC2")
    public int getStored() {
        return (int) Math.round(getEnergy() * general.TO_IC2);
    }

    @Override
    @Method(modid = "IC2")
    public int getCapacity() {
        return (int) Math.round(getMaxEnergy() * general.TO_IC2);
    }

    @Override
    @Method(modid = "IC2")
    public int getOutput() {
        return (int) Math.round(getMaxOutput() * general.TO_IC2);
    }

    @Override
    @Method(modid = "IC2")
    public double getDemandedEnergy() {
        return 0;
    }

    @Override
    @Method(modid = "IC2")
    public double getOfferedEnergy() {
        return Math.min(getEnergy(), getMaxOutput()) * general.TO_IC2;
    }

    @Override
    public boolean canReceiveEnergy(ForgeDirection side) {
        return false;
    }

    @Override
    @Method(modid = "IC2")
    public double getOutputEnergyUnitsPerTick() {
        return getMaxOutput() * general.TO_IC2;
    }

    @Override
    @Method(modid = "IC2")
    public double injectEnergy(ForgeDirection direction, double amount, double voltage) {
        return amount;
    }

    @Override
    @Method(modid = "IC2")
    public void drawEnergy(double amount) {
        if (structure != null) {
            double toDraw = Math.min(amount * general.FROM_IC2, getMaxOutput());
            setEnergy(Math.max(getEnergy() - toDraw, 0));
        }
    }

    @Override
    public double transferEnergyToAcceptor(ForgeDirection side, double amount) {
        return 0;
    }

    @Override
    public FluidTankInfo[] getTankInfo(ForgeDirection from) {
        return ((!worldObj.isRemote && structure != null)
                || (worldObj.isRemote && clientHasStructure))
            ? new FluidTankInfo[] { fluidTank.getInfo() }
            : PipeUtils.EMPTY;
    }

    @Override
    public int fill(ForgeDirection from, FluidStack resource, boolean doFill) {
        if (structure == null) {
            return 0;
        }
        if (resource.getFluid() == FluidRegistry.WATER) {
            return 0;
        }
        int filled = fluidTank.fill(resource, doFill);
        if (doFill) {
            structure.newSteamInput += filled;
        }

        if (filled < structure.getFluidCapacity() && structure.dumpMode != GasMode.IDLE) {
            filled = structure.getFluidCapacity();
        }

        return filled;
    }

    @Override
    public FluidStack drain(ForgeDirection from, FluidStack resource, boolean doDrain) {
        return null;
    }

    @Override
    public FluidStack drain(ForgeDirection from, int maxDrain, boolean doDrain) {
        return null;
    }

    @Override
    public boolean canFill(ForgeDirection from, Fluid fluid) {
        if (fluid == FluidRegistry.getFluid("steam")) {
            return (
                (!worldObj.isRemote && structure != null)
                || (!worldObj.isRemote && clientHasStructure)
            );
        }

        return false;
    }

    @Override
    public boolean canDrain(ForgeDirection from, Fluid fluid) {
        return false;
    }

    @Override
    public String getInventoryName() {
        return LangUtils.localize("gui.industrialTurbine");
    }
}
